@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 9889 2017-04-22 14:24:50Z David $
+# $Id: __init__.py 9891 2017-04-22 17:14:18Z David $
 #
 # Copyright (c) 2015 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: David $ (last)
-# $Date: 2017-04-22 22:24:50 +0800 (Sat, 22 Apr 2017) $
-# $Revision: 9889 $
+# $Date: 2017-04-23 01:14:18 +0800 (Sun, 23 Apr 2017) $
+# $Revision: 9891 $
 
 import os
 import sys
@@ -233,7 +233,7 @@ def populateEnvToSettings(keys, settingsDict, setter=None, evalKeys=None):
             
     return setKeys
     
-def attachSettings(settingsDict):
+def attachSettings(settingsDict, , warns=False, printDetail=True)):
     """
     Attach default settings to settings.py.
     It must be called in settings by passing locals().
@@ -266,10 +266,11 @@ def attachSettings(settingsDict):
         })
     )
     
-    print colored('SERVER_MODE', 'yellow', attrs=['bold']),
-    print colored(" = ", 'white', attrs=['bold']),
-    print colored(reversedServerModes[settingsDict['SERVER_MODE']], 
-                  'cyan', attrs=['bold'])    
+    if printDetail:
+        print colored('SERVER_MODE', 'yellow', attrs=['bold']),
+        print colored(" = ", 'white', attrs=['bold']),
+        print colored(reversedServerModes[settingsDict['SERVER_MODE']], 
+                    'cyan', attrs=['bold'])    
     #=========================================================================
 
     #=========================================================================
@@ -279,7 +280,6 @@ def attachSettings(settingsDict):
         
     debugDefaults = {
         'DEBUG': DEBUG,
-        'TEMPLATE_DEBUG': DEBUG,
     }
 
     update(settingsDict, debugDefaults)
@@ -301,8 +301,9 @@ def attachSettings(settingsDict):
         'APP_VERSION': '0.1.0.0000',
     }
 
-    warnWithout('APP_NAME', settingsDict, projectDefaults)
-    warnWithout('APP_VERSION', settingsDict, projectDefaults)
+    if warns:
+        warnWithout('APP_NAME', settingsDict, projectDefaults)
+        warnWithout('APP_VERSION', settingsDict, projectDefaults)
 
     update(settingsDict, projectDefaults)
 
@@ -321,13 +322,28 @@ def attachSettings(settingsDict):
     
     import socket
     
-    ALLOWED_HOSTS = (
-        'localhost', 
-        '127.0.0.1', 
-        '.nuwainfo.com', 
-        '.tracedig.com',
-        '.%s' % '.'.join(socket.gethostname().split('.')[1:]),
-    )
+    if settingsDict['SERVER_MODE'] == DEVELOPMENT:
+        ALLOWED_HOSTS = (
+            'localhost', 
+            '127.0.0.1',
+            '::1', 
+            # 1.8.16 required for stage server. See:
+            # https://docs.djangoproject.com/en/1.10/releases/
+            # 1.8.16/#dns-rebinding-vulnerability-when-debug-true
+            '.nuwainfo.com', 
+            '.tracedig.com',
+            '.%s' % '.'.join(socket.gethostname().split('.')[1:]),
+        )
+    else:
+        ALLOWED_HOSTS = ['127.0.0.1', 'localhost']
+        
+        if settingsDict['SITE_DOMAIN']:
+            siteDomain = settingsDict['SITE_DOMAIN']
+        else:
+            if warns:
+                warnWithout('SITE_DOMAIN', settingsDict, ALLOWED_HOSTS)
+        
+        ALLOWED_HOSTS.append(settingsDict['SITE_DOMAIN'])
 
     adminDefaults = {
         'ADMINS': ADMINS,
@@ -337,7 +353,8 @@ def attachSettings(settingsDict):
     
     require('SECRET_KEY', settingsDict)
 
-    warnWithout('ADMINS', settingsDict, adminDefaults)
+    if warns:
+        warnWithout('ADMINS', settingsDict, adminDefaults)
 
     update(settingsDict, adminDefaults)
     
@@ -447,6 +464,70 @@ def attachSettings(settingsDict):
         lambda k, v, s: s.update(databaseDefaults))
     #=========================================================================
 
+    #=========================================================================
+
+    # logging settings.
+    # =======================================================================
+    # https://docs.djangoproject.com/en/1.8/topics/logging/
+
+    # https://docs.sentry.io/clients/python/integrations/django/
+    # #integration-with-logging
+    # Please note the raven logging docs have bugs, you will get 
+    # ValueError: Unable to configure handler 'sentry': __init__() got an 
+    # unexpected keyword argument 'tags' 
+    # if you follow the docs.
+
+    loggingDefaults = {
+        'LOGGING': {
+            'version': 1,
+            'disable_existing_loggers': False,
+            'formatters': {
+                'verbose': {
+                    'format': ('%(asctime)s - %(levelname)s - %(message)s')
+                },
+            },
+            'handlers': {
+                'console': {
+                    'level': 'INFO',
+                    'class': 'logging.StreamHandler',
+                    'formatter': 'verbose'
+                },
+            },
+            'loggers': {}
+        }
+    }
+
+    if DEBUG:
+        loggingDefaults['LOGGING']['handlers'].update({
+            'debug_file': {
+                'level': 'DEBUG',
+                'class': 'logging.FileHandler',
+                'filename': os.path.join(BASE_DIR, 'debug.log'),
+            },        
+        })
+        loggingDefaults['LOGGING']['loggers'].update({
+            'django': {
+                'handlers': ['debug_file'],
+                'level': 'DEBUG',
+                'propagate': True,
+            },        
+        })
+
+
+    update(settingsDict, loggingDefaults)
+    
+    populateEnvToSettings((
+        'LOGGING',
+    ), settingsDict)
+    #=========================================================================
+    defaultForwardedHost = {
+        'USE_X_FORWARDED_HOST': True
+    }
+    update(settingsDict, defaultForwardedHost)
+    
+    populateEnvToSettings((
+        'USE_X_FORWARDED_HOST',
+    ), settingsDict)
     #=========================================================================
     # I18n.
     i18nDefaults = {
@@ -459,7 +540,7 @@ def attachSettings(settingsDict):
         'LANGUAGE_CODE': 'en-us',
         'USE_I18N': True,
         'LANGUAGES': (
-            ('zh-tw', 'Traditional Chinese'),
+            ('zh-hant', 'Traditional Chinese'),
             ('zh-cn', 'Simplified Chinese'),
             ('en-us', 'English'),
         ),
@@ -490,7 +571,7 @@ def attachSettings(settingsDict):
     if not DEBUG:   
         staticDefaults = {
             'STATIC_ROOT': os.path.join(projectPath, 'static'),
-            'STATIC_URL': staticURL,
+            'STATIC_URL': '/static/',
             'MEDIA_ROOT' : os.path.join(projectPath, 'media'),
             'MEDIA_URL' : '/media/',
         }
@@ -507,7 +588,7 @@ def attachSettings(settingsDict):
             'STATICFILES_DIRS': (
                 os.path.join(projectPath, 'static'),
             ),
-            'STATIC_URL': staticURL,
+            'STATIC_URL': '/static/',
             'MEDIA_ROOT' : os.path.join(projectPath, 'media'),
             'MEDIA_URL' : '/media/',
         }
@@ -544,8 +625,9 @@ def attachSettings(settingsDict):
     rootUrlDefaults = {
         'ROOT_URLCONF': '%s.urls' % projectName,
     }
-    
-    warnWithout('ROOT_URLCONF', settingsDict, rootUrlDefaults)
+
+    if warns:
+        warnWithout('ROOT_URLCONF', settingsDict, rootUrlDefaults)
     
     update(settingsDict, rootUrlDefaults)
     #=========================================================================
@@ -575,6 +657,7 @@ def attachSettings(settingsDict):
                 'django.contrib.auth.context_processors.auth',
                 'django.contrib.messages.context_processors.messages',
             ],
+            'debug': DEBUG,
         },             
     }
     
@@ -617,7 +700,8 @@ def attachSettings(settingsDict):
         ),
     }
 
-    warnWithout('INSTALLED_APPS', settingsDict, pluginsDefaults)
+    if warns:
+        warnWithout('INSTALLED_APPS', settingsDict, pluginsDefaults)
 
     update(settingsDict, pluginsDefaults)
     #=========================================================================
@@ -674,7 +758,8 @@ def attachSettings(settingsDict):
 
         Iuno.cloud.attachSettings(settingsDict, **attachCloudKws)
     else: # IUNO_CLOUD_ENABLE = False
-        warnPackageDisable('Iuno.cloud', settingsDict, iunoCloudDefaults)
+        if warns:
+            warnPackageDisable('Iuno.cloud', settingsDict, iunoCloudDefaults)
     #=========================================================================
     
     #=========================================================================
@@ -704,7 +789,8 @@ def attachSettings(settingsDict):
                         settingsDict['IUNO_MEMBER_YAHOO_LOGIN_ENABLE']
                     )
     else: # IUNO_MEMBER_ENABLE = False
-        warnPackageDisable('Iuno.member', settingsDict, iunoMemberDefaults)
+        if warns:
+            warnPackageDisable('Iuno.member', settingsDict, iunoMemberDefaults)
     #=========================================================================
 
     #=========================================================================
@@ -729,7 +815,8 @@ def attachSettings(settingsDict):
                     )
 
     else: # IUNO_SHOP_ENABLE = False
-        warnPackageDisable('Iuno.shop', settingsDict, iunoShopDefaults)
+        if warns:
+            warnPackageDisable('Iuno.shop', settingsDict, iunoShopDefaults)
     #=========================================================================
     
     #=========================================================================
@@ -742,6 +829,29 @@ def attachSettings(settingsDict):
     if settingsDict['SERVER_MODE'] == settingsDict['DEVELOPMENT']:
         Patch.attachDebugSettings(settingsDict)
     #=========================================================================
+    #=========================================================================
+    # debug tool bar & django extensions
+    try:
+        import debug_toolbar
+        import django_extensions
+    except:
+        pass
+    else:
+        if DEBUG:
+            settingsDict['INSTALLED_APPS'] += ['django_extensions',]
+            settingsDict['INSTALLED_APPS']+= ['debug_toolbar', ]
+            settingsDict['MIDDLEWARE_CLASSES'] += ("debug_toolbar.middleware.DebugToolbarMiddleware", )
+            defaultInternalIP = {
+                'INTERNAL_IPS': ['127.0.0.1'],
+            }
+            update(settingsDict, defaultInternalIP)
+            
+            populateEnvToSettings((
+                'INTERNAL_IPS',
+            ), settingsDict)
+
+    #=========================================================================
+
     
 # Attach Iuno Urls
 def attachIunoUrls(settings, urlpatterns):
@@ -758,3 +868,34 @@ def attachIunoUrls(settings, urlpatterns):
     urlpatterns += patterns('',
         (r'^i18n/', include('django.conf.urls.i18n')),
     )
+    
+    if settings.DEBUG == False:
+        urlpatterns += patterns('',
+            url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
+                'document_root': settings.MEDIA_ROOT,
+            })
+    )
+
+# Attach debug_tool_bar Urls
+def attachDebugToolBarUrls(settings, urlpatterns):
+    """
+    Attach debug_tool_bar related urlpatterns.
+    It must be called in urls.py by passing urlpatterns.
+
+    @param settings Django project's settings module.
+    @param urlpatterns urlpatterns in urls.py.
+    """
+    from django.conf.urls import url, include, patterns
+
+    if settings.DEBUG:
+        urlpatterns += patterns('',
+            url(r'^__debug__/', include(debug_toolbar.urls)),
+        )
+
+# detect Server mode
+def detectServerMode(settingsDict):
+    SERVER_MODE = (DEVELOPMENT if 'SERVER_MODE' not in os.environ else
+               settingsDict[(os.environ['SERVER_MODE'])])
+    DEBUG = True if SERVER_MODE not in (STAGE, PRODUCTION) else False
+
+    return SERVER_MODE, DEBUG
